package main

import (
	"bufio"
	"encoding/json"
	"fmt"
	"log"
	// Must be removed later
	"net/http"
	"strings"
	"time"

	"github.com/gorilla/websocket"
	"gitlab.com/khanzf/fedilogue/shared"
)

func StreamPleroma(endpoint string) {
	wss_url := "wss://" + endpoint + "/api/v1/streaming/?stream=public"
	var pleromaHeader shared.PleromaStreamHeader
	var newactivity shared.ReportActivity
	var err error

	for {
		var tries int
		var ws *websocket.Conn
		for tries = 0; tries < 10; tries++ {
			ws, _, err = websocket.DefaultDialer.Dial(wss_url, nil)
			if err != nil {
				continue
			}
			break
		}

		if tries == 10 {
			logWarn("Unable to connect to " + endpoint + " after 10 tries, exiting")
			return
		}

		for {
			_, p, err := ws.ReadMessage()
			if err != nil {
				log.Fatal(err)
				break
			}

			err = json.Unmarshal(p, &pleromaHeader)
			if err != nil {
				logDebug("Unable to parse data from " + endpoint + ", but still connected.")
				break
			}

			switch pleromaHeader.Event {
			case "update":
				err = json.Unmarshal([]byte(pleromaHeader.Payload), &newactivity)
				if err != nil {
					logDebug("Unable to parse data from " + endpoint + ", but still connected.")
					break
				}
				go check_activity(newactivity.Uri)
			default:
				continue
			}

		}

		ws.Close()
		time.Sleep(time.Minute * 10)

	}
}

func StreamMastodon(endpoint string, o *shared.RunningInstance) {
	stream_client := BuildClient(endpoint)

	var oauthData OAuth
	var retry bool

	api_timeline := "https://" + endpoint + "/api/v1/streaming/public"

	for {
		req, err := http.NewRequest("GET", api_timeline, nil)
		req.Header.Set("User-Agent", "Tusky")
		if err != nil {
			logFatal("Unable to create new request for " + endpoint + ", exiting.")
			return
		}

		for _, extaccount := range settings.Externalaccounts {
			if extaccount.Endpoint == endpoint {
				get_client(endpoint, o)

				err = get_client(endpoint, o)
				if err != nil {
					logWarn("Unable to register client: ", err)
				}

				oauthData, err = oauth_login(endpoint, o, extaccount.Username, extaccount.Password)
				if err != nil {
					logWarn("Unable to login: ", err)
					return
				}

				req.Header.Add("Authorization", oauthData.Access_token)

			}
		}

		var resp *http.Response

		for tries := 0; tries < 10; tries++ {
			resp, err = stream_client.Do(req)
			if err != nil {
				time.Sleep(time.Minute * 5)
				logWarn("Failure connecting to "+req.URL.Scheme+"://"+req.URL.Host+req.URL.Path+", attempt ", tries+1, ", sleeping for 5 minutes, ", err)
				continue
			}
			break
		}
		if err != nil {
			logErr("Unable to stream "+api_timeline+": ", err)
			return
		}
		defer resp.Body.Close()

		ri_mutex.Lock()
		m := runninginstances[endpoint]
		m.Status = shared.RUNNING
		m.LastRun = "Streaming"
		runninginstances[endpoint] = m
		ri_mutex.Unlock()

		s := bufio.NewScanner(resp.Body)
		var name string
		for s.Scan() {
			line := s.Text()
			token := strings.SplitN(line, ":", 2)
			var newactivity shared.ReportActivity
			if len(token) != 2 {
				continue
			}

			switch strings.TrimSpace(token[0]) {
			case "event":
				name = strings.TrimSpace(token[1])
				continue
			case "data":
				switch name {
				case "update":
					if len(token) >= 2 && len(token[1]) >= 2 {
						continue
					}
					jsondata := token[1][1:]
					err := json.Unmarshal([]byte(jsondata), &newactivity)
					if err != nil {
						logDebug("Unable to parse data from " + endpoint + ", but still connected.")
						continue
					}
					retry = true
				default:
					continue
				}
			default:
				continue
			}

			go check_activity(newactivity.Uri)

			matchset := re.FindStringSubmatch(newactivity.Uri)
			if matchset != nil {
				newinstance := matchset[1]
				go CheckInstance(newinstance, endpoint)
			}
		}
		if retry == true {
			time.Sleep(time.Minute * 10)
		} else {
			break
		}
	}
}
